/**********************************************************************\
**
**                         Copyright (c) 2007-2022 by
**                          HENGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a license and may be used and
**      copied only in accordance with the terms  of  such  license  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      Exo_Pmp_JobInfo.cpp
**  Facility:    S684015 - Hengstler Extendo Printer Driver for Windows XP/7/8/10
**  Component:   Print Monitor Plugin (Exo-PrintMonPlugin)
**  Version:     R10-V2.06-FINAL / 2022SEP28
**  Environment: Microsoft Windows XP/7/8/10 
**  Compiler:    Microsoft Visual Studio 2015
**  Abstract:    Module that manages the job info structure entry in the registry
**  Note:        -
**  History:     R1-V1.00 / 2007DEC21 / HOR / 1ST RELEASE 
**               -----------------------------------------------------
**               R2-V1.01 / 2008MAY31 / HOR / 2ND RELEASE 
**               -----------------------------------------------------
**               R3-V1.02 / 2010APR30 / HOR / 3RD RELEASE
**               -----------------------------------------------------
**               R4-V2.00a / 2011MAY26 / HOR / M-R4-004-HOR
**                  Complete printer package ported to VS2010, reviewed and adapted Windows 7 32+64 bit
**               R4-V2.00 / 2013JUL18 / HOR / 4TH RELEASE
**               -----------------------------------------------------
**               R5-V2.01 / 2015MAY07 / HOR / 5TH RELEASE
**               -----------------------------------------------------
**               R6-V2.02b / 2016MAY25 / HOR / M-R6-007-HOR
**                  Driver package ported to VS2015 and adapted to Windows 10.
**               R6-V2.02 / 2016MAY25 / HOR / 6TH RELEASE
**               -----------------------------------------------------
**               R7-V2.03 / 2017APR20 / HOR / 7TH RELEASE
**               -----------------------------------------------------
**               R8-V2.04 / 2018JUL02 / HOR / 8TH RELEASE
**               -----------------------------------------------------
**               R9-V2.05 / 2021SEP14 / HOR / 9TH RELEASE
**               -----------------------------------------------------
**               R10-V2.06 / 2022SEP28 / HOR / 10TH RELEASE
**               -----------------------------------------------------
**
\**********************************************************************/

/*
** INCLUDES
*/

#include "Exo_Pmp_System.h"
#include "Exo_PrintMonPlugin.h"
#include "Exo_Pmp_Interface.h"
#include "Exo_Pmp_RegistryAccess.h"
#include "Exo_Pmp_JobInfo.h"

/*
** COMPILER 
*/

/*
** MACROS
*/

/*
** TYPES
*/

/*
** FUNCTIONS
*/

static VOID Lo_InitJobInfoStructure      (PS_EXO_PMP_JOB_INFO psJobInfo);

/*
** VARIABLES
*/

//******************************************************************************************
// FUNCTION   Lo_InitJobInfoStructure
// ABSTRACT   Initializes the job info structure 
//******************************************************************************************

static VOID Lo_InitJobInfoStructure
   (PS_EXO_PMP_JOB_INFO psJobInfo)
{
   //-- Initialize job info structure 
   memset(psJobInfo, 0, sizeof(*psJobInfo));             // Clear complete structure
   psJobInfo->dwSize          = sizeof(*psJobInfo);      // Set size of this structure
   psJobInfo->ubRevision      = D_EXO_PMP_JI_REVISION;  // Set revision number (D_EXO_PMP_JI_REVISION)
   psJobInfo->dwTsLastUpdate  = 0;                       // Set timestamp of last update
   psJobInfo->dwJobId         = (DWORD) -1;              // Set job identifier to undefined (0xfff...)
   psJobInfo->dwJobStatus     = 0;                       // Set job status to invalid
   psJobInfo->dwPageCount     = 0;                       // Set page count to zero
}

//******************************************************************************************
// FUNCTION   Exo_Pmp_Ji_InitializeJobInfo
// ABSTRACT   Initializes job information in registry
//******************************************************************************************

bool Exo_Pmp_Ji_InitializeJobInfo
   (P_EXO_PLUGIN_DATA psData)
{
   bool                            fResult                    = false;                  // Return value, FALSE by default
   TS_EXO_PMP_JOB_INFO             sJobInfo;                                            // Job info data structure (registry)
   
   //-- Check parameter
   if (!psData || (psData->dwSructId != P_EXO_PLUGIN_DATA_ID))
      goto FAILURE;

   //-- Initialize job info structure 
   Lo_InitJobInfoStructure(&sJobInfo);

   //-- Set time stamp
   sJobInfo.dwTsLastUpdate = GetTickCount();

   //-- Write job info to registry
   if (!Exo_Pmp_Ra_WritePrinterDataToRegistry(psData, D_EXO_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo)))
      goto FAILURE;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}

//******************************************************************************************
// FUNCTION   Exo_Pmp_Ji_UpdateJobInfo_JobStarted
// ABSTRACT   Updates job info structure in registry at start of print job 
//******************************************************************************************

bool Exo_Pmp_Ji_UpdateJobInfo_JobStarted
   (P_EXO_PLUGIN_DATA psData, DWORD dwJobId)
{
   bool                 fResult    = false;  // Return value, FALSE by default
   TS_EXO_PMP_JOB_INFO  sJobInfo;            // Job info data structure (registry)
   
   //-- Check parameter
   if (!psData || (psData->dwSructId != P_EXO_PLUGIN_DATA_ID))
      goto FAILURE;

   //-- Initialize job info structure 
   Lo_InitJobInfoStructure(&sJobInfo);

   //-- Store job identifier
   sJobInfo.dwJobId = dwJobId;

   //-- Mark job as active
   sJobInfo.dwJobStatus = D_EXO_PMP_JI_JS_JOB_ACTIVE;  

   //-- Reset page number
   sJobInfo.dwPageCount = 0; 

   //-- Set time stamp
   sJobInfo.dwTsLastUpdate = GetTickCount();

   //-- Write job info to registry
   if (!Exo_Pmp_Ra_WritePrinterDataToRegistry(psData, D_EXO_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo)))
      goto FAILURE;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}

//******************************************************************************************
// FUNCTION   Exo_Pmp_Ji_UpdateJobInfo_JobCompleted
// ABSTRACT   Updates job info structure in registry at end of print job 
//******************************************************************************************

bool Exo_Pmp_Ji_UpdateJobInfo_JobCompleted
   (P_EXO_PLUGIN_DATA psData, DWORD dwJobId)
{
   bool                 fResult    = false;  // Return value, FALSE by default
   TS_EXO_PMP_JOB_INFO  sJobInfo;            // Job info data structure (registry)
   
   //-- Check parameter
   if (!psData || (psData->dwSructId != P_EXO_PLUGIN_DATA_ID))
      goto FAILURE;

   //-- Read job info from registry
   if (    !Exo_Pmp_Ra_ReadPrinterDataFromRegistry(psData, D_EXO_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo))
        || (sJobInfo.ubRevision != D_EXO_PMP_JI_REVISION)
      ) {

      //-- Setup values to default
      Lo_InitJobInfoStructure(&sJobInfo);
   }

   //-- Store job identifier
   sJobInfo.dwJobId = dwJobId;

   //-- Mark job as finished
   sJobInfo.dwJobStatus = D_EXO_PMP_JI_JS_JOB_FINISHED;  

   //-- Set time stamp
   sJobInfo.dwTsLastUpdate = GetTickCount();

   //-- Write job info to registry
   if (!Exo_Pmp_Ra_WritePrinterDataToRegistry(psData, D_EXO_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo)))
      goto FAILURE;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}

//******************************************************************************************
// FUNCTION   Exo_Pmp_Ji_UpdateJobInfo_JobCancelled
// ABSTRACT   Updates job info structure in registry in case the print job is cancelled 
//******************************************************************************************

bool Exo_Pmp_Ji_UpdateJobInfo_JobCancelled
   (P_EXO_PLUGIN_DATA psData, DWORD dwJobId)
{
   bool                 fResult    = false;  // Return value, FALSE by default
   TS_EXO_PMP_JOB_INFO  sJobInfo;            // Job info data structure (registry)
   
   //-- Check parameter
   if (!psData || (psData->dwSructId != P_EXO_PLUGIN_DATA_ID))
      goto FAILURE;

   //-- Read job info from registry
   if (    !Exo_Pmp_Ra_ReadPrinterDataFromRegistry(psData, D_EXO_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo))
        || (sJobInfo.ubRevision != D_EXO_PMP_JI_REVISION)
      ) {

      //-- Setup values to default
      Lo_InitJobInfoStructure(&sJobInfo);
   }

   //-- Store job identifier
   sJobInfo.dwJobId = dwJobId;

   //-- Mark job as finished
   sJobInfo.dwJobStatus = D_EXO_PMP_JI_JS_JOB_CANCELLED;  

   //-- Set time stamp
   sJobInfo.dwTsLastUpdate = GetTickCount();

   //-- Write job info to registry
   if (!Exo_Pmp_Ra_WritePrinterDataToRegistry(psData, D_EXO_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo)))
      goto FAILURE;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}

//******************************************************************************************
// FUNCTION   Exo_Pmp_Ji_UpdateJobInfo_PageCompleted
// ABSTRACT   Updates job info structure in registry at end of page 
//******************************************************************************************

bool Exo_Pmp_Ji_UpdateJobInfo_PageCompleted
   (P_EXO_PLUGIN_DATA psData, DWORD dwJobId)
{
   bool                 fResult    = false;  // Return value, FALSE by default
   TS_EXO_PMP_JOB_INFO  sJobInfo;            // Job info data structure (registry)
   
   //-- Check parameter
   if (!psData || (psData->dwSructId != P_EXO_PLUGIN_DATA_ID))
      goto FAILURE;

   //-- Read job info from registry
   if (    !Exo_Pmp_Ra_ReadPrinterDataFromRegistry(psData, D_EXO_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo))
        || (sJobInfo.ubRevision != D_EXO_PMP_JI_REVISION)
      ) {

      //-- Setup values to default
      Lo_InitJobInfoStructure(&sJobInfo);
   }

   //-- Increase page number, only if job identifier matches
   if (sJobInfo.dwJobId == dwJobId)
        sJobInfo.dwPageCount++; 
   else sJobInfo.dwPageCount = 0; 

   //-- Store job identifier
   sJobInfo.dwJobId = dwJobId;

   //-- Mark job as active
   sJobInfo.dwJobStatus = D_EXO_PMP_JI_JS_JOB_ACTIVE;  

   //-- Set time stamp
   sJobInfo.dwTsLastUpdate = GetTickCount();

   //-- Write job info to registry
   if (!Exo_Pmp_Ra_WritePrinterDataToRegistry(psData, D_EXO_PMP_REG_VALUE_PMP_JOB_INFO, (PBYTE) &sJobInfo, sizeof(sJobInfo)))
      goto FAILURE;

   //-- Success
   fResult = true;

   //-- All done
   goto EXIT;

FAILURE:
   ;

EXIT:

   //-- Return TRUE on success and FALSE otherwise
   return(fResult);
}
